@import url('https://fonts.googleapis.com/css?family=Cardo&display=swap');
@import url("https://fonts.googleapis.com/css?family=Open+Sans");
@import url('https://fonts.googleapis.com/css?family=Inconsolata&display=swap');

/* font families */

body {
    font-family: "Cardo", serif;
}

h1, h2, h3, h4, h5, h6, table, figure, [data-type=toc] {
    font-family: "Open Sans", sans-serif;
}

pre, code {
    font-family: "Inconsolata", monospace;
}

/* fon sizes */

body{font-size: 18px;}

h1{font-size: 35px;}
h2{font-size: 28px;}
h3{font-size: 21px;}
h4{font-size: 20px;}
h5{font-size: 19px;}
h6{font-size: 18px;}

caption, figcaption{
  font-size: 14px;
}

table{
  font-size: 14px;
}

pre{
  font-size: 14px;
}

/* text color */

body { color: #222; }

a:link, a:visited {
    color: navy;
    text-decoration: underline rgba(0,0,128,.25);
}

a:hover, a:active {
    color: red;
    text-decoration: underline red;
}

[data-type=toc] a {
    color: #222;
}

[data-type=toc] a.active {
    color: red;
}

a.section-mark {
    color: #DDD;
    text-decoration: none;
}

a[data-status=invalid] {
    color: maroon;
    text-decoration: line-through maroon;
}

/* background & border colors */

body {
    background-color: #EEE;
}

#layout-start{
    background-color: #FFF;
}

#layout-center main {
    background-color: #FFF;
    border-style: none solid;
    border-width: 1px;
    border-color: #DDD;
}

pre {
    border-radius: 5px;
    border-width: 1px;
    border-style: solid;
    border-color: #ebeafa;
    background-color: #fdfcfe;
}

code {
    background-color: rgba(27,31,35,.05);
    border-radius: 3px;
}

table, table th, table td {
    border-collapse: collapse;
    border: 1px solid #BBB;
}



/* link styles */

a {
    font-weight: bold;
}

[data-type=toc] a {
    font-weight: normal;
    text-decoration: none;
}

/* titles */

h1, h2, h3, h4, h5, h6 {
    margin: 1em 0 0.5em 0;
}

/* inline code */

code {
    padding: 2px 3px;
    margin: 0;
    white-space: nowrap;
}

/* block code */

pre {
    display: block;
    padding: 5px;
    margin: 1em;
}

pre p {
    margin: 0;
}
pre p + p {
    margin-top: 1em;
}
  
/* tables */

table {
    margin: 1em;
}
table th{
    font-weight: bold;
}
table th, table td {
    padding: 4px;
}

table ul,
table ol {
    margin: 0;
    padding-inline-start: 25px;
}

table p {
    margin: 0;
}

table pre,
table figure {
    margin: 0.5em;
}

/* figures */

figure {
    margin: 1em;
}

figure pre {
    margin: 0;
}

figcaption {
    margin-top: 0.5em;
}

/* paragraphs */
p {
    hyphens: auto;
    margin: 1em 0;
}

/* section marks */

a.section-mark {
    display: none;
}

h1:hover a.section-mark,
h2:hover a.section-mark,
h3:hover a.section-mark,
h4:hover a.section-mark,
h5:hover a.section-mark,
h6:hover a.section-mark {
    display: inline;
}

/* admonitions */

div[data-type=information] {
    border-color: #CDDDFF;
    border-style: solid;
    border-width: 1px;
    border-radius: 5px;
    padding: 0 15px 0 50px;
    background-color: #ebf1ff;
}

div[data-type=information]::before {
    content: '\24D8';
    position: absolute;
    font-size: 30px;
    color: #b8c6e5;
    margin-top: 10px;
    margin-left: -40px;
}

div[data-type=information] p {
    /* margin: 0; */
}


/* layouts */

[data-type=layout][data-dir=row] {
    display: flex;
    flex-direction: row;
    flex-wrap: nowrap;
    align-items: stretch;
}

[data-type=layout-item] {
    flex-grow: 1;
}

[data-type=layout] figure {
    margin-top: 0;
    margin-bottom: 0;
}